import { Injectable, NgModule, Optional } from '@angular/core';
import { CommonModule } from '@angular/common';
import { Headers, Http, HttpModule, RequestOptions } from '@angular/http';
import { utc } from 'moment';
import * as moment from 'moment';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/catch';

var LoggerConfig = (function () {
    function LoggerConfig() {
    }
    return LoggerConfig;
}());
var Levels = [
    'TRACE',
    'DEBUG',
    'INFO',
    'LOG',
    'WARN',
    'ERROR',
    'OFF'
];
var NGXLogger = (function () {
    /**
     * @param {?} http
     * @param {?} options
     */
    function NGXLogger(http, options) {
        this.http = http;
        this.options = options;
        this._isIE = navigator.userAgent.indexOf('MSIE') !== -1 || navigator.userAgent.match(/Trident\//) || navigator.userAgent.match(/Edge\//);
        this._serverLogLevelIdx = this._initLogLevel(this.options.serverLogLevel);
        this._clientLogLevelIdx = this._initLogLevel(this.options.level);
    }
    /**
     * @param {?} level
     * @return {?}
     */
    NGXLogger.prototype._initLogLevel = function (level) {
        level = level ? level.toUpperCase() : level;
        level = Levels.indexOf(level);
        return level === -1 ? Levels.indexOf('INFO') : level;
    };
    /**
     * @param {?} level
     * @param {?} message
     * @return {?}
     */
    NGXLogger.prototype._logOnServer = function (level, message) {
        var _this = this;
        if (!this.options.serverLoggingUrl)
            return;
        //if the user provides a serverLogLevel and the current level is than that do not log
        if (this._serverLogLevelIdx && Levels.indexOf(level) < this._serverLogLevelIdx)
            return;
        var /** @type {?} */ headers = new Headers({ 'Content-Type': 'application/json' });
        var /** @type {?} */ options = new RequestOptions({ headers: headers });
        this.http.post(this.options.serverLoggingUrl, { level: level, message: message }, options)
            .map(function (res) { return res.json(); })
            .catch(function (error) { return error; })
            .subscribe(function (res) { return null; }, function (error) { return _this._log('ERROR', 'FAILED TO LOG ON SERVER', false); });
    };
    /**
     * @param {?} level
     * @param {?} message
     * @return {?}
     */
    NGXLogger.prototype._logIE = function (level, message) {
        message = utc().format() + " [" + level + "] " + message;
        if (level === 'WARN') {
            console.warn(message);
        }
        else if (level === 'ERROR') {
            console.error(message);
        }
        else if (level === 'INFO') {
            console.info(message);
        }
        else {
            console.log(message);
        }
    };
    /**
     * @param {?} level
     * @param {?} message
     * @param {?} logOnServer
     * @return {?}
     */
    NGXLogger.prototype._log = function (level, message, logOnServer) {
        //if no message or the log level is less than the environ
        if (!message || Levels.indexOf(level) < this._clientLogLevelIdx)
            return;
        if (logOnServer) {
            this._logOnServer(level, message);
        }
        if (typeof message === 'object') {
            try {
                message = JSON.stringify(message, null, 2);
            }
            catch (e) {
                message = "circular object in message: " + message;
            }
        }
        // Coloring doesn't work in IE
        if (this._isIE) {
            return this._logIE(level, message);
        }
        var /** @type {?} */ color1;
        switch (level) {
            case 'TRACE':
                color1 = 'blue';
                break;
            case 'DEBUG':
                color1 = 'teal';
                break;
            case 'INFO':
            case 'LOG':
                color1 = 'gray';
                break;
            case 'WARN':
            case 'ERROR':
                color1 = 'red';
                break;
            case 'OFF':
            default:
                return;
        }
        console.log("%c" + utc().format() + " [" + level + "] %c" + message, "color:" + color1, 'color:black');
    };
    /**
     * @param {?} message
     * @return {?}
     */
    NGXLogger.prototype.trace = function (message) {
        this._log('TRACE', message, true);
    };
    /**
     * @param {?} message
     * @return {?}
     */
    NGXLogger.prototype.debug = function (message) {
        this._log('DEBUG', message, true);
    };
    /**
     * @param {?} message
     * @return {?}
     */
    NGXLogger.prototype.info = function (message) {
        this._log('INFO', message, true);
    };
    /**
     * @param {?} message
     * @return {?}
     */
    NGXLogger.prototype.log = function (message) {
        this._log('LOG', message, true);
    };
    /**
     * @param {?} message
     * @return {?}
     */
    NGXLogger.prototype.warn = function (message) {
        this._log('WARN', message, true);
    };
    /**
     * @param {?} message
     * @return {?}
     */
    NGXLogger.prototype.error = function (message) {
        this._log('ERROR', message, true);
    };
    return NGXLogger;
}());
NGXLogger.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
NGXLogger.ctorParameters = function () { return [
    { type: Http, },
    { type: LoggerConfig, decorators: [{ type: Optional },] },
]; };

var NGXLoggerMock = (function () {
    function NGXLoggerMock() {
    }
    /**
     * @param {?} message
     * @return {?}
     */
    NGXLoggerMock.prototype.trace = function (message) {
    };
    /**
     * @param {?} message
     * @return {?}
     */
    NGXLoggerMock.prototype.debug = function (message) {
    };
    /**
     * @param {?} message
     * @return {?}
     */
    NGXLoggerMock.prototype.info = function (message) {
    };
    /**
     * @param {?} message
     * @return {?}
     */
    NGXLoggerMock.prototype.log = function (message) {
    };
    /**
     * @param {?} message
     * @return {?}
     */
    NGXLoggerMock.prototype.warn = function (message) {
    };
    /**
     * @param {?} message
     * @return {?}
     */
    NGXLoggerMock.prototype.error = function (message) {
    };
    return NGXLoggerMock;
}());
NGXLoggerMock.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
NGXLoggerMock.ctorParameters = function () { return []; };

var LoggerModule = (function () {
    function LoggerModule() {
    }
    /**
     * @param {?} config
     * @return {?}
     */
    LoggerModule.forRoot = function (config) {
        return {
            ngModule: LoggerModule,
            providers: [
                { provide: LoggerConfig, useValue: config || {} },
                NGXLogger,
                NGXLoggerMock
            ]
        };
    };
    return LoggerModule;
}());
LoggerModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    HttpModule
                ]
            },] },
];
/**
 * @nocollapse
 */
LoggerModule.ctorParameters = function () { return []; };

export { LoggerModule, NGXLoggerMock, NGXLogger, LoggerConfig };
